import numpy as np
import joblib
import multiprocessing
from . import PACKAGE_PATH


# the main algorithm
def flip_image(img: np.ndarray) -> np.ndarray:
    """
    Flip an input image.

    This function uses an awesome algorithm to flip an input image.

    Parameters
    ----------
    img : np.ndarray
        The input image.

    Returns
    -------
    np.ndarray
        The flipped image.

    Examples
    --------
    >>> flip_image(np.array([1, 2], [3, 4]))
    """
    try:
        assert img.ndim == 2
        img_flipped = img[::-1, ::-1]
        return img_flipped
    except AssertionError:
        raise AssertionError("The input image is {}D not 2D!".format(img.ndim))


# a demo on how to read test image
def read_test_image():
    """
    Read test image.

    This function reads the test image associated with this package.

    Returns
    -------
    np.ndarray
        The test image.

    Examples
    --------
    >>> test_image = read_test_image()
    """
    fp_img = PACKAGE_PATH + "/data/test_image.txt"
    print("reading file {} ...".format(fp_img))
    return np.loadtxt(fp_img, dtype=int)


def flip_multiple_images_mp(imgs: list, n_jobs: int) -> list:
    """ parallel with multiprocessing """
    with multiprocessing.Pool(n_jobs) as p:
        results = p.map(flip_image, imgs)
    return results


def flip_multiple_images_jl(imgs: list, n_jobs: int) -> list:
    """ parallel with joblib """
    return joblib.Parallel(n_jobs=n_jobs)(
        joblib.delayed(flip_image)(img) for img in imgs
    )
